#include "xmegamp3_objects.h"
#include "xmegamp3_resources.h"

//--------------------- User code ---------------------//
// "void mp3();" - insert in end to _objects.h

#include "resources.h"

unsigned long file_size;
const BYTES_2_WRITE = 32;
const BUFFER_SIZE = 512;
char mp3_buffer[BUFFER_SIZE];
char volume_left, volume_right;

// Functions
void MP3_Init(void);
void MP3_Start(void);
void MP3_Test(char *test);

//- Imported from MP3_DRIVER.H --------------------------------------------------------------------
extern const char WRITE_CODE;
extern const char READ_CODE;

extern const char SCI_BASE_ADDR;
extern const char SCI_MODE_ADDR;
extern const char SCI_STATUS_ADDR;
extern const char SCI_BASS_ADDR;
extern const char SCI_CLOCKF_ADDR;
extern const char SCI_DECODE_TIME_ADDR;
extern const char SCI_AUDATA_ADDR;
extern const char SCI_WRAM_ADDR;
extern const char SCI_WRAMADDR_ADDR;
extern const char SCI_HDAT0_ADDR;
extern const char SCI_HDAT1_ADDR;
extern const char SCI_AIADDR_ADDR;
extern const char SCI_VOL_ADDR;
extern const char SCI_AICTRL0_ADDR;
extern const char SCI_AICTRL1_ADDR;
extern const char SCI_AICTRL2_ADDR;
extern const char SCI_AICTRL3_ADDR;

extern void MP3_SCI_Write(char address, unsigned int data_in);
extern void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer);
extern void MP3_SDI_Write(char data_);
extern void MP3_SDI_Write_32(char *data_);
extern void MP3_SDI_Write_256(char *data_);
extern void MP3_Set_Volume(char left,char right);

// !!!! This is only for testing proposes, remove for normal use !!!!
unsigned char ucMP3_run_test;
// !!!! This is only for testing proposes, remove for normal use !!!!

/**************************************************************************************************
* End of File
**************************************************************************************************/

/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
extern sfr sbit Mmc_Chip_Select;
extern sfr sbit MP3_CS;
extern sfr sbit MP3_RST;
extern sfr sbit BSYNC;
extern sfr sbit DREQ;

extern sfr sbit Mmc_Chip_Select_Direction;
extern sfr sbit MP3_CS_Direction;
extern sfr sbit MP3_RST_Direction;
extern sfr sbit BSYNC_Direction;
extern sfr sbit DREQ_Direction;

/**************************************************************************************************
* Function MP3_SCI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function writes one byte to MP3 SCI
* Input: register address in codec, data
* Output: Nothing
**************************************************************************************************/
void MP3_SCI_Write(char address, unsigned int data_in) {
  BSYNC = 1;

  MP3_CS = 0;                    // select MP3 SCI
  SPIC_Write(WRITE_CODE);
  SPIC_Write(address);
  SPIC_Write(Hi(data_in));       // high byte
  SPIC_Write(Lo(data_in));       // low byte
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SCI_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function reads words_count words from MP3 SCI
* Input: start address, word count to be read
* Output: words are stored to data_buffer
**************************************************************************************************/
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  MP3_CS = 0;                    // select MP3 SCI
  SPIC_Write(READ_CODE);
  SPIC_Write(start_address);

  while (words_count--) {        // read words_count words byte per byte
    temp = SPIC_Read(0);
    temp <<= 8;
    temp += SPIC_Read(0);
    *(data_buffer++) = temp;
  }
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SDI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write one byte to MP3 SDI
* Input: data to be writed
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write(char data_) {

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SPIC_Write(data_);
  BSYNC = 1;
}

/**************************************************************************************************
* Function MP3_SDI_Write_32
* -------------------------------------------------------------------------------------------------
* Overview: Function Write 32 bytes to MP3 SDI
* Input: data buffer
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write_32(char *data_) {
  char i;

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
  SPIC_Write(data_[i]);
  BSYNC = 1;
}

/**************************************************************************************************
* Function MP3_Set_Volume()
* -------------------------------------------------------------------------------------------------
* Overview: Function set volume on the left and right channel
* Input: left channel volume, right channel volume
* Output: Nothing
**************************************************************************************************/
void MP3_Set_Volume(char left, char right) {
  unsigned int volume;

  volume = (left<<8) + right;             // calculate value
  MP3_SCI_Write(SCI_VOL_ADDR, volume);    // Write value to VOL register
}

/**************************************************************************************************
* End of File
**************************************************************************************************/

/**************************************************************************************************
* MMC Chip Select connection
**************************************************************************************************/
// extern sbit Mmc_Chip_Select_Direction at PORTF_DIR.B5;
// extern sbit Mmc_Chip_Select           at PORTF_OUT.B5;

/**************************************************************************************************
* Serial Flash Chip Select connection
**************************************************************************************************/
sbit CS_Serial_Flash_bit           at PORTB_OUT.B7;
sbit CS_Serial_Flash_Direction_bit at PORTB_DIR.B7;

/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
sbit MP3_CS                    at PORTH_OUT.B6;
sbit MP3_RST                   at PORTH_OUT.B5;
sbit BSYNC                     at PORTH_OUT.B4;
sbit DREQ                      at PORTE_IN.B5;

sbit MP3_CS_Direction          at PORTH_DIR.B6;
sbit MP3_RST_Direction         at PORTH_DIR.B5;
sbit BSYNC_Direction           at PORTH_DIR.B4;
sbit DREQ_Direction            at PORTE_DIR.B5;
// VS1053E constants
const char WRITE_CODE           = 0x02;
const char READ_CODE            = 0x03;

const char SCI_BASE_ADDR        = 0x00;
const char SCI_MODE_ADDR        = 0x00;
const char SCI_STATUS_ADDR      = 0x01;
const char SCI_BASS_ADDR        = 0x02;
const char SCI_CLOCKF_ADDR      = 0x03;
const char SCI_DECODE_TIME_ADDR = 0x04;
const char SCI_AUDATA_ADDR      = 0x05;
const char SCI_WRAM_ADDR        = 0x06;
const char SCI_WRAMADDR_ADDR    = 0x07;
const char SCI_HDAT0_ADDR       = 0x08;
const char SCI_HDAT1_ADDR       = 0x09;
const char SCI_AIADDR_ADDR      = 0x0A;
const char SCI_VOL_ADDR         = 0x0B;
const char SCI_AICTRL0_ADDR     = 0x0C;
const char SCI_AICTRL1_ADDR     = 0x0D;
const char SCI_AICTRL2_ADDR     = 0x0E;
const char SCI_AICTRL3_ADDR     = 0x0F;

// Writes one byte to MP3 SCI
void MP3_SCI_Write(char address, unsigned int data_in);
// Reads words_count words from MP3 SCI
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer);
// Write one byte to MP3 SDI
void MP3_SDI_Write(char data_);
// Write 32 bytes to MP3 SDI
void MP3_SDI_Write_32(char *data_);
// Set volume
void MP3_Set_Volume(char left, char right);

/**************************************************************************************************
* End of File
**************************************************************************************************/



/**************************************************************************************************
* Function MC3_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MP3 VS1053B codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/

void MP3_Init(void)
{
  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000);   // default 12 288 000 Hz

  // Maximum volume is 0x00 and total silence is 0xFE.
  volume_left  = 0; //0x3F;
  volume_right = 0; //0x3F;
  MP3_Set_Volume(volume_left, volume_right);
}

/**************************************************************************************************
* Function MP3_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI to communicate with MP3 codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Start(void)
{
  // Disable other peripheral modules on the same SPI bus
  CS_Serial_Flash_Direction_bit = 1;
  CS_Serial_Flash_bit = 1;             // Disable Serial Flash module

  MP3_CS_Direction  = 1;               // Configure MP3_CS as output
  MP3_CS            = 1;               // Deselect MP3_CS
  MP3_RST_Direction = 1;               // Configure MP3_RST as output
  MP3_RST           = 1;               // Set MP3_RST pin

  DREQ_Direction    = 0;               // Configure DREQ as input
  BSYNC_Direction   = 1;               // Configure BSYNC as output
  BSYNC             = 0;               // Clear BSYNC
  BSYNC             = 1;               // Clear BSYNC

  // Initialize SPIC module
  // master_mode                   = _SPI_MASTER
  // Clock rate                    = _SPI_FCY_DIV16
  // SPI clock polarity and phase  = _SPI_CLK_LO_LEADING
  SPIC_Init_Advanced(_SPI_MASTER, _SPI_FCY_DIV16, _SPI_CLK_LO_LEADING);
  // TFT_BLED = 1;
  // TFT_Fill_Screen(CL_Black);
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);

  // TFT_Write_Text("1. Initializing VS1053B decoder interface", 30, 80);

  MP3_Init();
  Delay_ms(1000);
}

/**************************************************************************************************
* Function MP3_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the MP3 test
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void MP3_Test(char *test)
{
  unsigned long i;
  unsigned long roma;
  roma=0;

  // TFT_16bit_Fill_Screen(CL_Black_16bit);
  TFT_Set_Pen(CL_WHITE, 1);
  // TFT_16bit_Line(20, 220, 300, 220);
  // TFT_16bit_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED_16bit, FO_HORIZONTAL_16bit);
  // TFT_16bit_Set_Font(&Verdana12x13_Regular, CL_WHITE_16bit, FO_HORIZONTAL_16bit);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&Lucida_Console_KOI89x14_Bold, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("\xF2\xCF\xCD\xC1\xCE", 19, 25);
  TFT_Set_Font(&TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("MP3 Player for Xmega", 92, 5);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.ingineer.ru", 195, 25);

 while(1)
 {
  char mp3_filename[25][7] ={ "1.mp3","2.mp3","3.mp3","4.mp3","5.mp3","6.mp3","7.mp3","8.mp3","9.mp3","10.mp3","11.mp3","12.mp3","13.mp3","14.mp3","15.mp3","16.mp3","17.mp3","18.mp3","19.mp3","20.mp3","21.mp3","22.mp3","23.mp3","24.mp3","25.mp3"};

  // Reset error flag
  *test = 0;

  TFT_Set_Pen(CL_SILVER, 93);
  TFT_LIne(26,  126, 161,  126);
  TFT_Set_Font(&Lucida_Console_KOI89x14_Bold, CL_Black, FO_HORIZONTAL);
  TFT_Write_Text("1. \xEB\xCF\xC4\xC5\xCB \xF7\xC9\xD6\xD5", 30, 80);
  TFT_Write_Text("2. SD \xF7\xC9\xD6\xD5", 30, 100);
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&mp3_filename[roma], 0) ) {
      TFT_Write_Text("3. \xE6\xC1\xCA\xCC:", 30, 120);
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          //   procedure returns size of the file

      TFT_Write_Text("4. \xF0\xD2\xCF\xC9\xC7\xD2\xD9\xD7\xC1\xC5\xCD", 30, 140);
      TFT_Write_Text(&mp3_filename[roma], 100, 120);
      // send file blocks to MP3 SDI
      while (file_size > BUFFER_SIZE)
      {
        for (i=0; i<BUFFER_SIZE; i++)
        {
          Mmc_Fat_Read(mp3_buffer + i);
        }
        for (i=0; i<BUFFER_SIZE/BYTES_2_WRITE; i++) {
          MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
        }

        file_size -= BUFFER_SIZE;

    //Check_TP();
//        if (ucMP3_run_test == 0)return;
     }

      // send the rest of the file to MP3 SDI
      for (i=0; i<file_size; i++)
      {
        Mmc_Fat_Read(mp3_buffer + i);
      }

      for (i=0; i<file_size; i++)
      {
        MP3_SDI_Write(mp3_buffer[i]);
      }

      TFT_Write_Text("5. \xEB\xCF\xCE\xC5\xC3 \xE6\xC1\xCA\xCC\xC1", 30, 160);
    }
    else {

      TFT_Write_Text("3. \xE6\xC1\xCA\xCC\xC1 \xCE\xC5\xD4", 30, 120);
      *test = 2;
    }
  }
  else {
    TFT_Write_Text("3. SD \xCE\xC5 \xD7\xC9\xD6\xD5", 30, 120);
    *test = 2;
  }
  roma++;
  if (roma>25) { roma=0; }
 }
}

/**************************************************************************************************
* main function
**************************************************************************************************/
void mp3() {

  MP3_Start();
  MP3_Test(&ucMP3_run_test);


}
//----------------- End of User code ------------------//

// Event Handlers